#include <Python.h>
#include <string>
#include <vector>
#include "ngui.h"
#include "ngdata.h"
#include <iostream>

using namespace std;

static PyObject* ngsub_call( PyObject* self, PyObject* args )
{
  char* xrsl = NULL;
  char* cluster = NULL;
  int debug = UNDEFINED;
  
  if( !PyArg_ParseTuple( args, "z|iz", &xrsl, &debug, &cluster ) )
    return NULL;
  
  vector<string> xrslstrings;
  vector<string> clusterselect, clusterreject;

  if( strlen(xrsl) != 0 ) xrslstrings.push_back( xrsl );
  if( cluster != NULL && strlen(cluster) != 0 ) {
    if( cluster[0] == '-' ) {
      clusterreject.push_back( cluster+1 );
    } else {
      clusterselect.push_back( cluster );
    }
  }

  vector<string> dummy;
  string dummystring;

  int sts = ngsubxx( dummy, xrslstrings, clusterselect, clusterreject, dummy, dummystring, false, false, UNDEFINED, debug, true );
  return Py_BuildValue("i", sts);
};

static PyObject* ngstat_call( PyObject* self, PyObject* args )
{
  char* jobid = NULL;
  char* cluster = NULL;
  int debug = UNDEFINED;
  bool all = false;
  
  if( !PyArg_ParseTuple( args, "z|iz", &jobid, &debug, &cluster ) )
    return NULL;
  
  vector<string> jobs;
  vector<string> clusterselect, clusterreject;
  
  if( strlen(jobid) != 0 ) {
    if( !strcmp(jobid,"all") ) {
      all = true;
    } else {
      jobs.push_back( jobid );
    }
  }
  
  if(  cluster != NULL && strlen(cluster) != 0 ) {
    if( cluster[0] == '-' ) {
      clusterreject.push_back( cluster+1 );
    } else {
      clusterselect.push_back( cluster );
    }
  }
  
  vector<string> dummy;
  
  int sts = ngstatxx( jobs, dummy, clusterselect, clusterreject, dummy, all, dummy, false, false, UNDEFINED, debug, true );
  return Py_BuildValue("i", sts);
};

static PyObject* ngcat_call( PyObject* self, PyObject* args )
{
  char* jobid = NULL;
  int debug = UNDEFINED;
  bool all = false;
  
  if( !PyArg_ParseTuple( args, "z|i", &jobid, &debug ) )
    return NULL;
  
  vector<string> jobs;
  
  if( jobid != NULL && strlen(jobid) != 0 ) {
    if( !strcmp(jobid,"all") ) {
      all = true;
    } else {
      jobs.push_back( jobid );
    }
  }
  
  vector<string> dummy;
  
  int sts = ngcatxx( jobs, dummy, dummy, dummy, dummy, all, STDOUT, UNDEFINED, debug, false );
  return Py_BuildValue("i", sts);
};

static PyObject* ngclean_call( PyObject* self, PyObject* args )
{
  char* jobid = NULL;
  int debug = 0;
  bool all = false;
  
  if( !PyArg_ParseTuple( args, "z|iz", &jobid, &debug ) )
    return NULL;
  
  vector<string> jobs;
  
  if( strlen(jobid) != 0 ) {
    if( !strcmp(jobid,"all") ) {
      all = true;
    } else {
      jobs.push_back( jobid );
    }
  }
  
  vector<string> dummy;
  
  int sts = ngcleanxx( jobs, dummy, dummy, dummy, dummy, all, false, UNDEFINED, debug, true );
  return Py_BuildValue("i", sts);
};

static PyObject* ngsync_call( PyObject* self, PyObject* args )
{
  int debug = UNDEFINED;
  int force = 0;
  
  if( !PyArg_ParseTuple( args, "|ii", &force, &debug ) )
    return NULL;
  
  bool boolforce = false;
  if( force>0 ) boolforce = true;
  
  vector<string> dummy;  
  int sts = ngsyncxx( dummy, dummy, dummy, boolforce, UNDEFINED, debug, false );
  return Py_BuildValue("i", sts);
};

static PyObject* ngkill_call( PyObject* self, PyObject* args )
{
  char* jobid = NULL;
  int debug = UNDEFINED;
  bool all = false;
  
  if( !PyArg_ParseTuple( args, "z|iz", &jobid, &debug ) )
    return NULL;
  
  vector<string> jobs;
  
  if( jobid != NULL && strlen(jobid) != 0 ) {
    if( !strcmp(jobid,"all") ) {
      all = true;
    } else {
      jobs.push_back( jobid );
    }
  }
  
  vector<string> dummy;
  
  int sts = ngkillxx( jobs, dummy, dummy, dummy, dummy, all, UNDEFINED, debug, true );
  return Py_BuildValue("i", sts);
};

static PyObject* ngrenew_call( PyObject* self, PyObject* args )
{
  char* jobid = NULL;
  int debug = UNDEFINED;
  bool all = false;
  
  if( !PyArg_ParseTuple( args, "z|iz", &jobid, &debug ) )
    return NULL;
  
  vector<string> jobs;
  
  if( jobid != NULL && strlen(jobid) != 0 ) {
    if( !strcmp(jobid,"all") ) {
      all = true;
    } else {
      jobs.push_back( jobid );
    }
  }
  
  vector<string> dummy;
  
  int sts = ngrenewxx( jobs, dummy, dummy, dummy, dummy, all, UNDEFINED, debug, true );
  return Py_BuildValue("i", sts);
};

static PyObject* ngget_call( PyObject* self, PyObject* args )
{
  char* jobid = NULL;
  int debug = UNDEFINED;
  int keep = 0;
  string directory;

  if( !PyArg_ParseTuple( args, "z|zii", &jobid, &directory, &keep, &debug ) )
    return NULL;
  
  vector<string> jobs;
  
  bool all = false;

  if( jobid != NULL && strlen(jobid) != 0 ) {
    if( !strcmp(jobid,"all") ) {
      all = true;
    } else {
      jobs.push_back( jobid );
    }
  }

  bool keepbool = false;
  if( keep>0 ) keepbool = true;

  vector<string> dummy;
  
  int sts = nggetxx( jobs, dummy, dummy, dummy, dummy, all, directory, false, keepbool, UNDEFINED, debug, true );
  return Py_BuildValue("i", sts);
};

static PyObject* ngcopy_call( PyObject* self, PyObject* args )
{
  char* source = NULL;
  char* dest = NULL;
  int debug = UNDEFINED;

  if( !PyArg_ParseTuple(args, "zz|i", &source, &dest, &debug) )
    return NULL;

  if( debug>3 ) debug = 3;
  if( debug<UNDEFINED ) debug = UNDEFINED;

  int argc = 3;
  string level;
  if( debug>0 ) {
    argc += 2;
    level += (char)(48+debug);
  }

  char* argv[argc+1];
  argv[0] = "ngcopy";
  argv[argc-2] = source;
  argv[argc-1] = dest;
  argv[argc] = NULL;

  if( debug>0 ) {
    argv[1] = "-d";
    argv[2] = (char*)level.c_str();
  }    

  int sts = ngcopy( argc, argv );
  return Py_BuildValue("i", sts);
};

static PyObject* ngremove_call( PyObject* self, PyObject* args )
{
  char* dest = NULL;
  int debug = UNDEFINED;

  if( !PyArg_ParseTuple(args, "z|i", &dest, &debug) )
    return NULL;

  if( debug>3 ) debug = 3;
  if( debug<UNDEFINED ) debug = UNDEFINED;

  int argc = 2;
  string level;
  if( debug>0 ) {
    argc += 2;
    level += (char)(48+debug);
  }

  char* argv[argc+1];
  argv[0] = "ngremove";
  argv[argc-1] = dest;
  argv[argc] = NULL;

  if( debug>0 ) {
    argv[1] = "-d";
    argv[2] = (char*)level.c_str();
  }    

  for( int i=0; i<argc; i++) {
    printf("%s\n",argv[i]);
  }

  int sts = ngremove( argc, argv );
  return Py_BuildValue("i", sts);
};

static PyObject* ngls_call( PyObject* self, PyObject* args )
{
  char* dest = NULL;
  char* option = NULL;
  int debug = UNDEFINED;

  if( !PyArg_ParseTuple(args, "z|zi", &dest, &option, &debug) )
    return NULL;

  if( debug>3 ) debug = 3;
  if( debug<UNDEFINED ) debug = UNDEFINED;

  int argc = 2;
  string level;
  if( debug>0 ) {
    level += (char)(48+debug);
  }    

  bool optionl = false;
  bool optionL = false;
  if( option != NULL && strlen(option)>0 ) {
    for( int i=0; i<strlen(option); i++) {
      if( option[i]=='l' && optionl == false ) {
	optionl = true;
	argc++;
      }
      if( option[i]=='L' && optionL == false ) {
	optionL = true;
	argc++;
      }
    }
  }

  char* argv[argc+1];
  int tmp = 1;
  if( debug>0 ) {
    argv[tmp] = "-d";
    argv[tmp+1] = (char*)level.c_str();
    tmp += 2;
  }
  if( optionl == true ) {
    argv[tmp] = "-l";
    tmp++;
  }
  if( optionL == true ) {
    argv[tmp] = "-L";
    tmp++;
  }

  argv[0] = "ngremove";
  argv[argc-1] = dest;
  argv[argc] = NULL;

  for( int i=0; i<argc; i++) {
    printf("%s\n",argv[i]);
  }

  int sts = ngls( argc, argv );
  return Py_BuildValue("i", sts);
};

static PyObject* ngacl_call( PyObject* self, PyObject* args )
{
  char* option = NULL;
  char* dest = NULL;
  int debug = UNDEFINED;

  if( !PyArg_ParseTuple(args, "zz|i", &option, &dest, &debug) )
    return NULL;

  if( debug>3 ) debug = 3;
  if( debug<UNDEFINED ) debug = UNDEFINED;

  int argc = 3;
  string level;
  if( debug>0 ) {
    argc += 2;
    level += (char)(48+debug);
  }

  char* argv[argc+1];
  argv[0] = "ngcopy";
  argv[argc-2] = option;
  argv[argc-1] = dest;
  argv[argc] = NULL;

  if( debug>0 ) {
    argv[1] = "-d";
    argv[2] = (char*)level.c_str();
  }    

  int sts = ngcopy( argc, argv );
  return Py_BuildValue("i", sts);
};

static PyMethodDef NorduGridMethods[] = {
  {"ngkill", ngkill_call, METH_VARARGS, "Execute ngkill."},
  {"ngrenew", ngrenew_call, METH_VARARGS, "Execute ngrenew."},
  {"ngsub", ngsub_call, METH_VARARGS, "Execute ngsub."},
  {"ngcat", ngcat_call, METH_VARARGS, "Execute ngcat."},
  {"ngclean", ngclean_call, METH_VARARGS, "Execute ngclean."},
  {"ngget", ngget_call, METH_VARARGS, "Execute ngget."},
  {"ngstat", ngstat_call, METH_VARARGS, "Execute ngstat."},
  {"ngsync", ngsync_call, METH_VARARGS, "Execute ngsync."},
  {"ngls", ngls_call, METH_VARARGS, "Execute ngls."},
  {"ngacl", ngacl_call, METH_VARARGS, "Execute ngacl."},
  {"ngcopy", ngcopy_call, METH_VARARGS, "Execute ngcopy."},
  {"ngremove", ngremove_call, METH_VARARGS, "Execute ngremove."},
  {NULL, NULL, 0, NULL}
};

extern "C" {

void initnordugrid( void )
{
  (void)Py_InitModule("nordugrid", NorduGridMethods);
};

}
