#include <iostream>
#include <fstream>

#include "qstring.h"
#include "qwidget.h"
#include "qpopupmenu.h"
#include "qmenubar.h"
#include "qlineedit.h"
#include "qfiledialog.h"
#include "qtabwidget.h"
#include "qapplication.h"
#include "qstatusbar.h"

#include "MainWindow.h"
#include "JobSubWindow.h"
#include "JobStatWindow.h"

using namespace std;

MainWindow::MainWindow( QWidget* parent, const char* name ) : QMainWindow( parent, name )
{
  jobWindow = new JobSubWindow( );
  jobStats = new JobStatWindow( );

  tabs = new QTabWidget( this );
  tabs->setTabShape( QTabWidget::Triangular );
  tabs->addTab( jobWindow, "Job-submission" );
  tabs->addTab( jobStats, "Job-status" );
  tabs->show();
  setCentralWidget( tabs );

  QPopupMenu *fileMenu = new QPopupMenu( this );
  fileMenu->insertItem( "Load xrsl", this, SLOT( load( ) ), CTRL+Key_L );  
  fileMenu->insertItem( "Save xrsl", this, SLOT( save( ) ), CTRL+Key_S );  
  fileMenu->insertSeparator();
  fileMenu->insertItem( "Quit", qApp, SLOT(quit()), CTRL+Key_Q );

  QPopupMenu *subMenu = new QPopupMenu( this );
  subMenu->insertItem( "Submit xrsl", jobWindow, SLOT( submit( ) ), CTRL+Key_X );  
  subMenu->insertItem( "Update cluster-list", jobWindow, SLOT( submit( ) ), CTRL+Key_U );  

  menuBar()->insertItem( "&File", fileMenu );
  menuBar()->insertItem( "&Actions", subMenu );

  lineEdit = new QLineEdit( statusBar() );
  lineEdit->clear();
  lineEdit->setMaxLength( 300 );
  lineEdit->setReadOnly( true );
  lineEdit->setFrame( false );
  statusBar()->addWidget( lineEdit );

  connect( jobWindow, SIGNAL( linemessage( const QString& ) ), lineEdit, SLOT( setText( const QString& ) ) );
  connect( jobStats, SIGNAL( linemessage( const QString& ) ), lineEdit, SLOT( setText( const QString& ) ) );

  QString text("Program initialized...");
  lineEdit->setText( text );
}

MainWindow::~MainWindow( )
{
  delete tabs;
}

void MainWindow::load( )
{
  string filename = (const char*)QFileDialog::getOpenFileName( ".", "Xrsl-files (*.xrsl)", this, "load xrsl-file", "Choose a file");

  if( filename.empty() == true ) {
    lineEdit->setText( "No filename given..." );
    return;
  }

  ifstream file( filename.c_str() );
  if( !file ) {
    lineEdit->setText( "Could not open file..." );
    return;
  }

  string filecontents, line;
  while( getline( file, line ) ) {
    filecontents += line + '\n';
  }
  filecontents.resize( filecontents.size()-1 );
  
  jobWindow->setTextArea( QString( filecontents.c_str() ) );

  int slash = filename.rfind('/');
  string cutfilename = filename.substr( slash+1, filename.size()-slash-1 );

  string statusline( "Read file " + cutfilename + "..." );
  lineEdit->setText( QString( statusline.c_str() ) );
  return;
}

void MainWindow::save( )
{
  string filename = (const char*)QFileDialog::getSaveFileName( ".", "Xrsl-files (*.xrsl)", this, "save xrsl-file", "Choose a file");

  if( filename.empty() == true ) {
    lineEdit->setText( "No filename given..." );
    return;
  }

  ofstream file( filename.c_str() );
  file << jobWindow->returnXrsl();

  int slash = filename.rfind('/');
  string cutfilename = filename.substr( slash+1, filename.size()-slash-1 );
  
  string statusline( "Saved file " + cutfilename + "..." );
  lineEdit->setText( QString( statusline.c_str() ) );
  return;
}
